# GenerateBridge.tcl
# Units: kip, in
# Kevin Mackie, 2002/01/29
# mackie@ce.berkeley.edu
# University of California Berkeley
#
# Generates bridge model using input parameters, used by all
# bridge simulations.  This variation is for single column,
# single bent bridges with abutments and pile shafts
#
# input 1. skew bridge degree of skewness (degrees)
#	      2. L span of each segment (inch)
#	      3. LoH span to column height ratio
#	      4. fy steel strength (ksi)
#       5. fc concrete strength (ksi)
#       6. rho_s longitudinal reinforcement ratio
#       7. DcDc column width to superstructure depth ratio
#       8. ksoil soil properties corresponding to soil groups (B,C,D)
#       9. wt gravity load (weight) on columns (% above current load)
#      10. rhos_trans volume of transverse steel to volume of confined core
#      11. deck model (0=elastic, 1=NL)
#      12. gap value of abutment gap (in)
#      13. abut whether or not to include the abutment in analysis (0=no,1,2,3)
#
# output - all elements defining the model

proc GenerateBridge {skew L LoH fy fc rho_s DcDs ksoil wt rhos_trans deck gap abut} {
# constants
set g 386.4
set pi [expr acos(-1.0)]
set gamma [expr 150.0/1000.0/1728.0]
set poisson 0.2
set subspans 5.0
set deck_depth 7.0
set deck_width 36.0
set cover 1.5
set cracked_factor 0.50
set torsion_factor 0.25
set trans_bar 6.0

# build important quantities from input paramters
set Ds [expr $deck_depth*12.0]
set Dcol [expr $Ds*$DcDs]
set Ht [expr $L/$LoH]
set As [expr $rho_s*$pi/4.0*pow($Dcol,2)]
set li [expr $L/$subspans]
set Hp [expr $Ht*1.75]
set dh [expr $Hp/6.0]
if { $gap > 99 } {
   set gapl 100.0
   set gapt 100.0
} else {
   set gapl $gap
   set gapt 0.0
}
set skew [expr $skew*$pi/180.0]
if { $deck > 1 } {
   set deck 0
}

# Define the model builder
model BasicBuilder -ndm 3 -ndf 6

# calculate cross-section properties for superstructure
set E [expr 57000.0*sqrt($fc*1000.0)/1000.0]
set A 8960.0
set Iz [expr 10299150.0 * $cracked_factor]
set Iy [expr 164711680.0 * $cracked_factor]
set J [expr 31020600.0 * $torsion_factor]
set G [expr $E/2.0/(1.0+$poisson)]
set GJ [expr $G*$J]

# Torsional stiffness column
set Jc $pi*pow($Dcol*12.0,4)/32.0
set GJc [expr $G*$Jc]

# Calculate loads and masses
set DL [expr $gamma*$A*$li+$gamma*$A*$li*$wt]
set m1 [expr $DL/$g]
set m2 [expr $m1/2.0]
set mR [expr $m1*pow($li,2)/12.0]
set mQ [expr $m2*pow($li/2.0,2)/12.0]
set mX [expr $m1*pow($deck_width*12.0,2)/12.0]
set mY [expr $m2*pow($deck_width*12.0,2)/12.0]
set mc [expr $pi*pow($Dcol,2)/4.0*$gamma/$g*$Ht/4.0]
set mm [expr $mc*2.0]
set m3 [expr $mc+$m2]
set mp [expr $pi*pow($Dcol,2)/4.0*$gamma/$g*$dh]
set mL [expr $m1/$li]

# Define structural nodes
#    tag       X         Y   Z   -mass   X    Y    Z   rX   rY   rZ
node  1   $L  0          0       -mass  $mc  $mc  $mc  0    0    0
if { $deck == 0 } {
node  3        0         0  $Ht  -mass  $m2  $m2  $m2  $mY  $mQ  $mQ
node  4  [expr $li*1.0]  0  $Ht  -mass  $m1  $m1  $m1  $mX  $mR  $mR
node  5  [expr $li*2.0]  0  $Ht  -mass  $m1  $m1  $m1  $mX  $mR  $mR
node  6  [expr $li*3.0]  0  $Ht  -mass  $m1  $m1  $m1  $mX  $mR  $mR
node  7  [expr $li*4.0]  0  $Ht  -mass  $m1  $m1  $m1  $mX  $mR  $mR
node  8        $L        0  $Ht  -mass  $m3  $m3  $m3  $mX  $mR  $mR
node  9  [expr $li*6.0]  0  $Ht  -mass  $m1  $m1  $m1  $mX  $mR  $mR
node 10  [expr $li*7.0]  0  $Ht  -mass  $m1  $m1  $m1  $mX  $mR  $mR
node 11  [expr $li*8.0]  0  $Ht  -mass  $m1  $m1  $m1  $mX  $mR  $mR
node 12  [expr $li*9.0]  0  $Ht  -mass  $m1  $m1  $m1  $mX  $mR  $mR
node 13  [expr $L*2.0]   0  $Ht  -mass  $m2  $m2  $m2  $mY  $mQ  $mQ
} else {
node  3        0         0  $Ht  -mass   0    0    0   $mY  $mQ  $mQ
node  8        $L        0  $Ht  -mass   0    0    0   $mX  $mR  $mR
node 13  [expr $L*2.0]   0  $Ht  -mass   0    0    0   $mY  $mQ  $mQ
}

# nodes for pile shafts below grade
node 20  $L  0  [expr -$Hp]         -mass  $mp $mp $mp   0    0    0
node 22  $L  0  [expr -$Hp+1.0*$dh] -mass  $mp $mp $mp   0    0    0
node 24  $L  0  [expr -$Hp+2.0*$dh] -mass  $mp $mp $mp   0    0    0
node 26  $L  0  [expr -$Hp+3.0*$dh] -mass  $mp $mp $mp   0    0    0
node 28  $L  0  [expr -$Hp+4.0*$dh] -mass  $mp $mp $mp   0    0    0
node 30  $L  0  [expr -$Hp+5.0*$dh] -mass  $mp $mp $mp   0    0    0

# nodes for abutment springs and gaps
set w1 [expr $deck_width*12.0/4.0]
set w2 [expr $w1*2.0]
set y1 [expr $w1*cos($skew)]
set y2 [expr $w2*cos($skew)]
set x1 [expr $w1*sin($skew)]
set x2 [expr $w2*sin($skew)]

if { $abut == 0 } {
   node 41   $x1  $y1  $Ht
   node 42   $x2  $y2  $Ht
   node 43  -$x1 -$y1  $Ht
   node 44  -$x2 -$y2  $Ht
   node 51  [expr $L*2.0+$x1]  $y1  $Ht
   node 52  [expr $L*2.0+$x2]  $y2  $Ht
   node 53  [expr $L*2.0-$x1] -$y1  $Ht
   node 54  [expr $L*2.0-$x2] -$y2  $Ht
} else {
   node 40    0    0   $Ht
   node 42   $x2  $y2  $Ht
   node 44  -$x2 -$y2  $Ht
   node 45   $x1  $y1  $Ht
   node 46   $x2  $y2  $Ht
   node 47  -$x1 -$y1  $Ht
   node 48  -$x2 -$y2  $Ht
   node 50  [expr $L*2.0]       0   $Ht
   node 52  [expr $L*2.0+$x2]  $y2  $Ht
   node 54  [expr $L*2.0-$x2] -$y2  $Ht
   node 55  [expr $L*2.0+$x1]  $y1  $Ht
   node 56  [expr $L*2.0+$x2]  $y2  $Ht
   node 57  [expr $L*2.0-$x1] -$y1  $Ht
   node 58  [expr $L*2.0-$x2] -$y2  $Ht
   if { $abut > 0 && $abut <= 10} {
      node 41   $x1  $y1  $Ht
      node 43  -$x1 -$y1  $Ht
      node 51  [expr $L*2.0+$x1]  $y1  $Ht
      node 53  [expr $L*2.0-$x1] -$y1  $Ht 
   } else {
      puts "ERROR: only abutment options 0-8,10 allowed"
   }
}
 
# springs for pile p-y SSI
node 60  $L  0   0
node 62  $L  0  [expr -$Hp+5.0*$dh]
node 64  $L  0  [expr -$Hp+4.0*$dh]
node 66  $L  0  [expr -$Hp+3.0*$dh]
node 68  $L  0  [expr -$Hp+2.0*$dh]
node 70  $L  0  [expr -$Hp+1.0*$dh]
node 72  $L  0  [expr -$Hp]

# Single point constraints
#   node DX DY DZ RX RY RZ
fix   1   0  0  0  0  0  1
fix  20   0  0  1  0  0  0

if { $abut == 0 } {
fix  42   0  0  1  0  0  0
fix  44   0  0  1  0  0  0
fix  52   0  0  1  0  0  0
fix  54   0  0  1  0  0  0
} else {
fix  40   1  1  1  1  1  1
fix  45   1  1  1  1  1  1
fix  46   1  1  1  1  1  1
fix  47   1  1  1  1  1  1
fix  48   1  1  1  1  1  1
fix  50   1  1  1  1  1  1
fix  55   1  1  1  1  1  1
fix  56   1  1  1  1  1  1
fix  57   1  1  1  1  1  1
fix  58   1  1  1  1  1  1
}

fix  60   1  1  1  1  1  1
fix  62   1  1  1  1  1  1
fix  64   1  1  1  1  1  1
fix  66   1  1  1  1  1  1
fix  68   1  1  1  1  1  1
fix  70   1  1  1  1  1  1
fix  72   1  1  1  1  1  1

# Define uniaxial materials

# Mander concrete model for confined and unconfined concrete
# requires rhos_trans = 4 Asp / (ds s)
# where Asp = cross sectional area of spiral reinforcement
#       ds = center to center diameter inside spirals
#       s = center to center spacing of spiral
set epss 0.005
set eps0 0.002
set epsu 0.02
set f1 [expr 1.0/2.0*$rhos_trans*$fy]
set dcs [expr $Dcol-2.0*$cover-($trans_bar/8.0)]
set rho_cc [expr $As/($pi/4.0*pow($dcs,2))]
set spacing [expr $pi*pow($trans_bar/8.0,2)/$rhos_trans/$dcs]
set ke [expr (1.0-($spacing-$trans_bar/8.0)/2.0/$dcs)/(1.0-$rho_cc)]
set fcc [expr $fc*(-1.254+2.254*sqrt(1.0+7.94*$ke*$f1/$fc)-2.0*$ke*$f1/$fc)]
if { $fcc > 2.23*$fc } {
   set fcc [expr 2.23*$fc]
}
set epsc [expr $eps0*(1.0+5.0*($fcc/$fc-1.0))]
set ecr [expr $E/($E-$fcc/$epsc)]
set fcu [expr $fcc*$epsu/$epsc*$ecr/($ecr-1.0+pow($epsu/$epsc,$ecr))]

# Cover concrete
#                          tag -f'c  -epsco  -f'cu  -epscu
uniaxialMaterial Concrete01 1  -$fc  -$eps0    0.0  -$epss
# Core concrete
uniaxialMaterial Concrete01 2 -$fcc  -$epsc  -$fcu  -$epsu  -min -$epsu -max $epsu

# Steel model with softening
source Iwan.tcl
set Ef 29000.0
set Ehf [expr $Ef*0.015]
Iwan [expr $Ef-$Ehf] [expr $fy/($Ef-$Ehf)] 10 4
#                          tag    E   epsY
uniaxialMaterial ElasticPP  5   $Ehf  0.08
uniaxialMaterial Elastic    6  [expr -$Ehf/2.0]
uniaxialMaterial Parallel   3  4 5 6  -min -0.12 -max 0.12

# force-deformation relationship for abutment springs and gap element
#                             tag  K (k/in)  uy|fy   gap (in)
# bearing pads (eps_u = 3.5 eps_y)
uniaxialMaterial ElasticPP     31     10     2.00
if { $gapl > 99 } {
   # don't want jump after pads fail
   # we're pushing to failure, so their contribution isn't important
   # for column quantities, yield, etc
   uniaxialMaterial Parallel      41   31  -min -0.2 -max 0.2
} elseif { $gapl > 7.0 } {
   uniaxialMaterial Parallel      41   31  -min -7.0 -max 7.0
} else {
   uniaxialMaterial Parallel      41   31  -min -$gapl -max $gapl
}

# abutment springs and fy values are 1/2 of actual because 
# of parallel configuration of 2 springs in abutment array
# (for longitudinal direction only)
switch $abut {
   0 {
      # no spring stiffnesses required, roller supports
   }
   1 {
      # Goel, Chopra, Caltrans stiffnesses
   }
   2 {
      # Maroney backbone stiffnesses
   }
   3 {
      # Wilson and Tan stiffnesses
   }
   4 {
      # Makris embankments and stiffnesses
   }
   5 {
      # Aschheim embankments and stiffnesses
   }
   6 {
      # Siddhartan
   }
   7 {
      # Maroney with soil mass
   }
   8 {
      # Makris with soil mass
   }
   10 {
      # Arbitrary input mass and stiffness
   }
   default {
      puts "ERROR: only abutment options 0-8,10 allowed"
   }
}

# calculate p-y spring stiffnesses based on input parameters
switch $ksoil {
   A {
   }
   B {
   }
   C {
   }
   D {
   }
   default {
      puts "ERROR: only soil types USGS A, B, C, D accepted"
   }
}
set gam_prime [expr $gam_prime/1728.0]
set kp [expr sqrt($Dcol/24.0)*$ki]

# force-deformation relationships for p-y springs
source PYspring.tcl
set counter 0
while {$counter < 7} {
   set cur_depth [expr $counter*$dh+$dh/2.0]
   if { $counter == 0 || $counter == 6 } {
      set trib_plen $dh/2.0
   } else {
      set trib_plen $dh
   }
   set pus [expr ($C1*$cur_depth+$C2*$Dcol)*$gam_prime*$cur_depth]
   set pud [expr $C3*$Dcol*$gam_prime*$cur_depth]
   if { $pus <= $pud } {
      set pu $pus
   } else {
      set pu $pud
   }
   PYspring $pu $kp $cur_depth $trib_plen [expr 60+$counter]
   set counter [expr $counter+1]
}

# Geometric transformation
#                tag  vecxz
geomTransf LinearWithPDelta 1   1 0 0

# Geometric transformation
#                tag  vecxz
geomTransf Linear 2   1 1 0

# Column section
#    Dcol - diameter of column (in)
#    rebar - rebar size (in 1/8's of an inch)
#    ri - inner radius of the section
#    ro - overall (outer) radius of the section
#    coreID - material tag for the core patch
#    coverID - material tag for the cover patches
#    steelID - material tag for the reinforcing steel
#    numBars - number of reinforcing bars around the section perimeter
#    barArea - cross-sectional area of each reinforcing bar
#    nfCoreR - number of radial divisions in the core (number of "rings")
#    nfCoreT - number of theta divisions in the core (number of "wedges")
#    nfCoverR - number of radial divisions in the cover
#    nfCoverT - number of theta divisions in the cover
set rebar 11.0
switch $rebar {
   18.0 {
      set db 2.257
   }
   14.0 {
      set db 1.693
   }
   11.0 {
      set db 1.410
   }
   10.0 {
      set db 1.270
   }
   9.0 {
      set db 1.128
   }
   default { 
      set db [expr $rebar/8.0]
   }
}
set barArea [expr $pi/4.0*pow($db,2)]
set numBars [expr int(ceil($As/$barArea))]
set nfCoreT 12
set nfCoreR 8
set nfCoverT 12
set nfCoverR 2
set ro [expr $Dcol/2.0]
set ri [expr $ro-$cover-($db/2.0)-($trans_bar/8.0)]
set total_conc_fibers [expr $nfCoreT*$nfCoreR+$nfCoverT*$nfCoverR]

set mat_file [open "fiber_numbers" "w"]
puts $mat_file "$total_conc_fibers  $numBars"
close $mat_file

section fiberSec 1 {
    # Define the core patch
    patch circ 2 $nfCoreT $nfCoreR 0 0 0 $ri 0 360

    # Define the cover patch
    patch circ 1 $nfCoverT $nfCoverR 0 0 $ri $ro 0 360

    # Determine angle increment between bars
    set theta [expr 360.0/$numBars]

    # Define the reinforcing layer
    layer circ 3 $numBars $barArea 0 0 $ri $theta 360
  }

if { $deck == 1 } {
   # import UCFyber nonlinear superstructure section
   section UCFiber 3 boxgirder.g3
}

# Linear elastic torsion
uniaxialMaterial Elastic 98 $GJ
uniaxialMaterial Elastic 99 $GJc

# Attach torsion to the RC section
#                 tag uniTag uniCode       secTag
section Aggregator 2    99      T      -section 1
if { $deck == 1 } {
section Aggregator 4    98      T      -section 3
}

# Large values to provide rigid elements
set Ei 900000.0
set Ai 900000.0
set Ii 900000000.0
set Ji 900000000.0
set Gi 900000.0

# Number of integration points
set nPc 5
set nPp 4

# Define elements

# Columns
#                           tag ndI ndJ   nPts  secID transf
element nonlinearBeamColumn  1   1   8    $nPc    2     1

# Pile shafts
element nonlinearBeamColumn 20  20  22    $nPp   2     1
element nonlinearBeamColumn 22  22  24    $nPp   2     1
element nonlinearBeamColumn 24  24  26    $nPp   2     1
element nonlinearBeamColumn 26  26  28    $nPp   2     1
element nonlinearBeamColumn 28  28  30    $nPp   2     1
element nonlinearBeamColumn 30  30   1    $nPp   2     1

# Beams
if { $deck == 0 } {
#                         tag ndI ndJ  A   E   G   Jx   Iy   Iz  transf
element elasticBeamColumn  3   3   4   $A  $E  $G  $J  $Iy  $Iz    2    
element elasticBeamColumn  4   4   5   $A  $E  $G  $J  $Iy  $Iz    2
element elasticBeamColumn  5   5   6   $A  $E  $G  $J  $Iy  $Iz    2
element elasticBeamColumn  6   6   7   $A  $E  $G  $J  $Iy  $Iz    2
element elasticBeamColumn  7   7   8   $A  $E  $G  $J  $Iy  $Iz    2
element elasticBeamColumn  8   8   9   $A  $E  $G  $J  $Iy  $Iz    2
element elasticBeamColumn  9   9  10   $A  $E  $G  $J  $Iy  $Iz    2
element elasticBeamColumn 10  10  11   $A  $E  $G  $J  $Iy  $Iz    2
element elasticBeamColumn 11  11  12   $A  $E  $G  $J  $Iy  $Iz    2
element elasticBeamColumn 12  12  13   $A  $E  $G  $J  $Iy  $Iz    2
} else {
#                           tag ndI ndJ   nPts  secID transf
element nonlinearBeamColumn  3   3   8    $nPc    4     2  -mass $mL
element nonlinearBeamColumn  8   8  13    $nPc    4     2  -mass $mL
}

if { $abut == 0 } {
# rigid links at abutments
#                         tag ndI ndJ  A   E   G   Jx   Iy   Iz  transf
element elasticBeamColumn 100  3   41  $Ai $Ei $Gi $Ji $Ii  $Ii    2
element elasticBeamColumn 101  41  42  $Ai $Ei $Gi $Ji $Ii  $Ii    2
element elasticBeamColumn 102  3   43  $Ai $Ei $Gi $Ji $Ii  $Ii    2
element elasticBeamColumn 103  43  44  $Ai $Ei $Gi $Ji $Ii  $Ii    2
element elasticBeamColumn 104  13  51  $Ai $Ei $Gi $Ji $Ii  $Ii    2
element elasticBeamColumn 105  51  52  $Ai $Ei $Gi $Ji $Ii  $Ii    2
element elasticBeamColumn 106  13  53  $Ai $Ei $Gi $Ji $Ii  $Ii    2
element elasticBeamColumn 107  53  54  $Ai $Ei $Gi $Ji $Ii  $Ii    2
} else {
# rigid links at abutments
#                         tag ndI ndJ  A   E   G   Jx   Iy   Iz  transf
element elasticBeamColumn 100  3   41  $Ai $Ei $Gi $Ji $Ii  $Ii    2
element elasticBeamColumn 101  41  42  $Ai $Ei $Gi $Ji $Ii  $Ii    2
element elasticBeamColumn 102  3   43  $Ai $Ei $Gi $Ji $Ii  $Ii    2
element elasticBeamColumn 103  43  44  $Ai $Ei $Gi $Ji $Ii  $Ii    2
element elasticBeamColumn 104  13  51  $Ai $Ei $Gi $Ji $Ii  $Ii    2
element elasticBeamColumn 105  51  52  $Ai $Ei $Gi $Ji $Ii  $Ii    2
element elasticBeamColumn 106  13  53  $Ai $Ei $Gi $Ji $Ii  $Ii    2
element elasticBeamColumn 107  53  54  $Ai $Ei $Gi $Ji $Ii  $Ii    2

source skew.tcl

# gap elements
#                   id  ndI ndJ        mats       dirs        orientation
element zeroLength  40  46   42  -mat  42 43  -dir  1 3 
element zeroLength  41  48   44  -mat  42 43  -dir  1 3  
element zeroLength  42  52   56  -mat  42 43  -dir  1 3  
element zeroLength  43  54   58  -mat  42 43  -dir  1 3  

# abutment springs
#                   id  ndI ndJ        mats       dirs
element zeroLength  44  40    3  -mat  44  -dir  2    
element zeroLength  45  45   41  -mat  41 41  -dir  1 2
element zeroLength  46  47   43  -mat  41 41  -dir  1 2 
element zeroLength  47  13   50  -mat  44  -dir  2      
element zeroLength  48  51   55  -mat  41 41  -dir  1 2
element zeroLength  49  53   57  -mat  41 41  -dir  1 2 
}

# pile p-y springs
element zeroLength  60   1   60  -mat  60 60  -dir  1 2
element zeroLength  62  30   62  -mat  61 61  -dir  1 2
element zeroLength  64  28   64  -mat  62 62  -dir  1 2
element zeroLength  66  26   66  -mat  63 63  -dir  1 2
element zeroLength  68  24   68  -mat  64 64  -dir  1 2
element zeroLength  70  22   70  -mat  65 65  -dir  1 2
element zeroLength  72  20   72  -mat  66 66  -dir  1 2

# Constant gravity loads
set col_load [expr $Ht*$gamma*$pi/4.0*pow($Dcol,2)]
if { $deck == 0 } {
   pattern Plain 1 Linear {
      #    node  FX  FY       FZ          MX  MY  MZ
      load   1  0.0 0.0 [expr -$col_load] 0.0 0.0 0.0 -const
      load   3  0.0 0.0 [expr -$DL/2.0]   0.0 0.0 0.0 -const
      load   4  0.0 0.0 [expr -$DL]       0.0 0.0 0.0 -const
      load   5  0.0 0.0 [expr -$DL]       0.0 0.0 0.0 -const
      load   6  0.0 0.0 [expr -$DL]       0.0 0.0 0.0 -const
      load   7  0.0 0.0 [expr -$DL]       0.0 0.0 0.0 -const
      load   8  0.0 0.0 [expr -$DL]       0.0 0.0 0.0 -const
      load   9  0.0 0.0 [expr -$DL]       0.0 0.0 0.0 -const
      load  10  0.0 0.0 [expr -$DL]       0.0 0.0 0.0 -const
      load  11  0.0 0.0 [expr -$DL]       0.0 0.0 0.0 -const
      load  12  0.0 0.0 [expr -$DL]       0.0 0.0 0.0 -const
      load  13  0.0 0.0 [expr -$DL/2.0]   0.0 0.0 0.0 -const
   }
} else {
   pattern Plain 1 Linear {
      #    node  FX  FY        FZ             MX  MY  MZ
      load   1  0.0 0.0 [expr -$col_load]     0.0 0.0 0.0 -const
      load   3  0.0 0.0 [expr -$mL*$L/2.0*$g] 0.0 0.0 0.0 -const
      load   8  0.0 0.0 [expr -$mL*$L*$g]     0.0 0.0 0.0 -const
      load  13  0.0 0.0 [expr -$mL*$L/2.0*$g] 0.0 0.0 0.0 -const
   }
}

# end of bridge generation script
}
